package com.holmos.webtest.constvalue;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.util.Properties;

import com.holmos.webtest.log.MyLogger;
import com.holmos.webtest.utils.HolmosPropertiesUtils;

/**
 * Holmos框架配置文件用到的常量,加载配置文件规则<br>
 * (1)优先级如下，如果C盘有holmosconf目录，则加载C盘下面的配置文件，其他地方的无效<br>
 * (2)如果C盘没有，则加载用户根目录下面的路径，这个方便大家在linux系统下面进行配置<br>
 * (3)如果用户根目录没有，则加载使用者工程所在根目录下的holmosconf目录下的配置文件<br>
 * (4)如使用者工程根目录也没有，则使用classpath路径的holmosconf目录下的配置文件<br>
 *
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public class ConfigConstValue {
	
	private static MyLogger logger = MyLogger.getLogger(ConfigConstValue.class);
	
	private static String CONFNAME="holmosconf";
	
	private static String CONFDIR;
	
	public static String UPLOADFILE;
	
	public static String CLOSEWINDOW;
	
	public static String DOWNLOADFILE;
	
	public static String IEDRIVER;
	
	public static String CHROMEDRIVER;
	
	public static String CSSPROPERTIESCONFIG;
	
	public static String IMGCSSPROPERTIESCONFIG;
	
	public static String HOLMOSCONFFILE;
	
	/**配置的从用户当前工作目录到指定的包的根目录之间的路径信息，这个主要是为了参数化测试做准备*/
	public static String CURRENTWORKTOPACKAGEPATH = "\\src\\test\\java\\";
	
	/**默认等待一次的时间*/
	public static int defaulwaitMiliSeconds=50;
	
	/**默认等待次数,默认等待一个元素加载5秒*/
	public static int defaultWaitCount=100;
	
	/**等待加载次数,默认等待加载10s*/
	public static int waitCount=200;
	
	/**默认等待页面加载时间,默认10s*/
	public static int waitPageLoadTime=10000;
	
	/**autoIt3x.dll文件地址*/
	public static String AUTOITDLLPATH;
	
	/**用来指明Holmos框架的数据源路径前缀配置，是否要添加当前的测试类所在包的包路径*/
	public static final String DATA_SOURCE_PACKAGE_PREFIX_USEABLE="Holmos.data.source.package.prefix.useable";
	
	/**设定Holmos框架数据源路径的前缀,不设置或设置为""，则认为没有前缀，否则是有前缀的*/
	public static final String DATA_SOURCE_PREFIX_VALUE="Holmos.data.source.prefix.value";

	private static void initResourcesConf(){
		File conf=new File(ClassLoader.getSystemResource("").getPath().substring(1)+CONFNAME);
		if(conf.exists()&&conf.isDirectory())
			CONFDIR=conf.getAbsolutePath();
	}
	
	private static void initProjectBaseConf(){
		File conf=new File(System.getProperty("user.dir")+"\\"+CONFNAME);
		if(conf.exists()&&conf.isDirectory())
			CONFDIR=conf.getAbsolutePath();
	}
	
	private static void initUserDirBaseConf(){
		File conf=new File(System.getProperty("user.home")+"\\"+CONFNAME);
		if(conf.exists()&&conf.isDirectory())
			CONFDIR=conf.getAbsolutePath();
	}
	
	private static void initCConf(){
		File conf=new File("C:\\"+CONFNAME);
		if(conf.exists()&&conf.isDirectory())
			CONFDIR=conf.getAbsolutePath();
	}
	
	private static void initHolmosConf(){
		initResourcesConf();
		initProjectBaseConf();
		initUserDirBaseConf();
		initCConf();
		if(null == CONFDIR){
			logger.warn("holmos的配置文件没有加载成功!将使用默认配置!\n" +
					"您需要自行配置chromedriver和IEDDriverServer:配置方法请参考:http://wd.holmos.com/?/question/1015\n" +
					"您将无法正常使用以下功能:\n" +
					"(1)web页面的上传和下载功能" +
					"(2)无法正常用Holmos来进行页面样式的校验\n" +
					"其他所有的功能，均不造成任何影响!请您放心使用，如有问题，请找银龙!");
		}else{
			UPLOADFILE=CONFDIR+"\\autoItScripts\\upload.exe";
			DOWNLOADFILE=CONFDIR+"\\autoItScripts\\download.exe";
			CLOSEWINDOW=CONFDIR+"\\autoItScripts\\closewindow.exe";
			IEDRIVER=CONFDIR+"\\driverServers\\IEDriverServer.exe";
			CHROMEDRIVER=CONFDIR+"\\driverServers\\chromedriver.exe";
			CSSPROPERTIESCONFIG=CONFDIR+"\\cssProperties\\css_comman_property_config.properties";
			IMGCSSPROPERTIESCONFIG=CONFDIR+"\\cssProperties\\css_img_property_config.properties";
			HOLMOSCONFFILE=CONFDIR+"\\holmosConf.properties";
		}
	}
	
	static{
		try {
			initHolmosConf();
			if(null != HOLMOSCONFFILE){
				Properties properties=HolmosPropertiesUtils.getPropertyInfo(HOLMOSCONFFILE);
				defaulwaitMiliSeconds=HolmosPropertiesUtils.getInt(properties, "defaulwaitMiliSeconds");
				defaultWaitCount=HolmosPropertiesUtils.getInt(properties, "defaultWaitCount");
				waitCount=HolmosPropertiesUtils.getInt(properties, "waitCount");
				waitPageLoadTime=HolmosPropertiesUtils.getInt(properties, "waitPageLoadTime");
				CURRENTWORKTOPACKAGEPATH=HolmosPropertiesUtils.getValue(properties, "currentWorkToPackageDir");
				HOLMOSCONFFILE = java.net.URLDecoder.decode(HOLMOSCONFFILE,"utf8");
				UPLOADFILE=java.net.URLDecoder.decode(UPLOADFILE,"utf8");
				DOWNLOADFILE=java.net.URLDecoder.decode(DOWNLOADFILE,"utf8");
				IEDRIVER=java.net.URLDecoder.decode(IEDRIVER,"utf8");
				CHROMEDRIVER=java.net.URLDecoder.decode(CHROMEDRIVER,"utf8");
				CSSPROPERTIESCONFIG=java.net.URLDecoder.decode(CSSPROPERTIESCONFIG,"utf8");
				IMGCSSPROPERTIESCONFIG=java.net.URLDecoder.decode(IMGCSSPROPERTIESCONFIG,"utf8");
				CURRENTWORKTOPACKAGEPATH=java.net.URLDecoder.decode(CURRENTWORKTOPACKAGEPATH,"utf8");
				System.setProperty("webdriver.ie.driver", IEDRIVER);
				System.setProperty("webdriver.chrome.driver", CHROMEDRIVER);
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
	}
}
