package com.holmos.webtest.element.locator;

import java.util.ArrayList;

import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;

import com.holmos.webtest.element.Element;

/**保存locator链路上的信息，包含两个元素
 * （1）一个节点的locator信息
 * （2）下一个节点的对象
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public abstract class AbstractElement {
	
	public abstract Locator getLocator();
	public abstract String getComment();
	public abstract void setComment(String comment);
	public abstract WebElement getElement();
	public abstract void setElement(WebElement element);
	public abstract String getLocatorCurrent();
	public abstract void setLocatorCurrent(String locatorCurrent);
	public abstract boolean isExist();
	public abstract LocatorChain getInfoChain();
	public abstract void setWholeComment(String currentComment);
	
	/**
	 * 返回此元素的子元素，返回第一个被匹配的元素
	 * */
	public Element findElement(Locator locator){
		Element element = new Element("");
		element.setLocator(locator);
		if(null != this.getInfoChain())
			element.getInfoChain().addParentNode(this.getInfoChain().getInfoNodes());
		element.getInfoChain().addNode(this);
		return element;
	}
	
	public ArrayList<Element> findElements(Locator locator){
		ArrayList<Element> elements = new ArrayList<Element>();
		if(isExist()){
			WebElement myself = getElement();
			ArrayList<WebElement> webElements = getWebElements(myself, locator);
			int index = 0;
			for(WebElement webElement : webElements){
				Element element = new Element(Integer.toString(index));
				element.setCreatedByFindElements(true);
				element.setElement(webElement);
				elements.add(element);
			}
		}
		return elements;
	}
	
	private ArrayList<WebElement> getWebElements(WebElement myself,Locator locator){
		ArrayList<WebElement> webElements = null;
		if(webElements == null && locator.getLocatorById() != null){
			webElements = (ArrayList<WebElement>) myself.findElements(By.id(locator.getLocatorById()));
		}if(webElements == null && locator.getLocatorByName() != null){
			webElements = (ArrayList<WebElement>) myself.findElements(By.name(locator.getLocatorByName()));
		}if(webElements == null && locator.getLocatorByXpath() != null){
			webElements = (ArrayList<WebElement>) myself.findElements(By.xpath(locator.getLocatorByXpath()));
		}if(webElements == null && locator.getLocatorByCSS() != null){
			webElements = (ArrayList<WebElement>) myself.findElements(By.cssSelector(locator.getLocatorById()));
		}if(webElements == null && locator.getLocatorByClass() != null){
			webElements = (ArrayList<WebElement>) myself.findElements(By.className(locator.getLocatorByClass()));
		}if(webElements == null && locator.getLocatorByTagName() != null){
			webElements = (ArrayList<WebElement>) myself.findElements(By.tagName(locator.getLocatorByTagName()));
		}if(webElements == null && locator.getLocatorByTagName() != null){
			webElements = (ArrayList<WebElement>) myself.findElements(By.linkText(locator.getLocatorByTagName()));
		}if(webElements == null && locator.getLocatorByPartialLinkText() != null){
			webElements = (ArrayList<WebElement>) myself.findElements(By.partialLinkText(locator.getLocatorByPartialLinkText()));
		}
		return webElements;
	}
}
