package com.holmos.webtest.element.tool;

import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;

import com.holmos.webtest.Allocator;
import com.holmos.webtest.constvalue.ConfigConstValue;
import com.holmos.webtest.constvalue.ConstValue;
import com.holmos.webtest.element.locator.AbstractElement;
import com.holmos.webtest.element.locator.Locator;
import com.holmos.webtest.element.locator.LocatorFinder;
import com.holmos.webtest.struct.Collection;
import com.holmos.webtest.struct.Frame;
import com.holmos.webtest.struct.Page;
/**
 * 用来判断webdriver web元素是否存在的类，采用的是策略模式
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public class WebDriverElementExist extends WebElementExist{
	/**
	 * 元素的判断采用策略模式实现，此为WebDriver类型的数据判断
	 * @param webElement 待判断的元素
	 * */
	public WebDriverElementExist(AbstractElement webElement){
		super(webElement);
	}
	
	@Override
	public boolean isElementExist(int waitCount){
		if(isElementExistForCheckOnce())
			return true;
		initComment();
		AbstractElement lastNode=infoChain.getInfoNodes().get(infoChain.getInfoNodes().size()-1);
		if(lastNode instanceof Page || lastNode instanceof Frame){
			return findOneLevelElement(waitCount);
		}else{
			return findMuiltiLevelElement(waitCount);
		}
	}
	@Override
	public boolean isElementExistForCheckOnce(){
		AbstractElement lastNode=infoChain.getInfoNodes().get(infoChain.getInfoNodes().size()-1);
		if(lastNode instanceof Page || lastNode instanceof Frame){
			return findOneLevelElement(1);
		}else{
			return findMuiltiLevelElement(1);
		}
	}
	private boolean findMuiltiLevelElement(int waitCount) {
		int startLevel=infoChain.getInfoNodes().size()-1;
		for(;startLevel>1;startLevel--){
			if(infoChain.getInfoNodes().get(startLevel) instanceof Frame){
				//找到Frame层级，跳到下一层
				startLevel++;
				break;
			}
		}
		WebDriver driverTemp=(WebDriver) Allocator.getInstance().currentWindow.getDriver().getEngine();
		WebElement elementTemp=null;
		elementTemp=findElement(new LocatorFinder(driverTemp), infoChain.getInfoNodes().get(startLevel));
		if(elementTemp==null)return false;
		for(int i=startLevel+1;i<infoChain.getInfoNodes().size();i++){
			for(int j=0;j<waitCount;j++){
				elementTemp=findElement(new LocatorFinder(elementTemp), infoChain.getInfoNodes().get(startLevel));//这个地方对于Collection需要修改
				if(elementTemp!=null)break;
			}
			if(elementTemp==null)return false;
		}
		
		elementTemp=findElement(new LocatorFinder(elementTemp),element);
		if(elementTemp!=null){
			element.setElement(elementTemp);return true;
		}
		return false;
	}
	/**判断单层元素是否存在*/
	private boolean findOneLevelElement(int waitCount) {
		LocatorFinder finder=new LocatorFinder(Allocator.getInstance().currentWindow.getDriver().getEngine());
		WebElement elementTemp;
		for(int i=0;i<waitCount;i++){
			elementTemp=findElement(finder,element);
			if(elementTemp!=null){
				element.setElement(elementTemp);
				return true;
			}
		}
		return false;
	}
	/**查找多级别web层元素，判断其是否存在*/
	private WebElement findElement(LocatorFinder finder,AbstractElement webElement){
		WebElement element = null;
		if(webElement instanceof Collection){
			int index=((Collection)webElement).getIndex();
			element=findCollection(finder, webElement, index);
		}else{
			element=findElementOnly(finder, webElement);
		}
		return element;
	}
	@Override
	public int getListElementSize(){
		if(isElementExist(ConfigConstValue.defaultWaitCount)){
			AbstractElement lastNode=infoChain.getInfoNodes().get(infoChain.getInfoNodes().size()-1);
			if(lastNode instanceof Page || lastNode instanceof Frame){
				return getOneLevelListElementSize();
			}else{
				return getMuiltiLevelListElementSize();
			}
		}
		return ConstValue.ERROR;
	}
	public int getCollectionSize(){
		return getListElementSize();
	}
	private int getMuiltiLevelListElementSize() {
		int startLevel=infoChain.getInfoNodes().size()-1;
		for(;startLevel>1;startLevel--){
			if(infoChain.getInfoNodes().get(startLevel) instanceof Frame){
				//找到Frame层级，跳到下一层
				startLevel++;
				break;
			}
		}
		WebDriver driverTemp=(WebDriver) Allocator.getInstance().currentWindow.getDriver().getEngine();
		WebElement elementTemp=null;
		elementTemp=findElement(new LocatorFinder(driverTemp), infoChain.getInfoNodes().get(startLevel));
		if(elementTemp==null)return -1;
		for(int i=startLevel+1;i<infoChain.getInfoNodes().size();i++){
			elementTemp=findElement(new LocatorFinder(elementTemp), infoChain.getInfoNodes().get(startLevel));
		}
		return getListElementSize(new LocatorFinder(elementTemp),element);
	}

	private int getOneLevelListElementSize() {
		LocatorFinder finder=new LocatorFinder(Allocator.getInstance().currentWindow.getDriver().getEngine());
		return getListElementSize(finder,element);
	}

	private int getListElementSize(LocatorFinder finder,AbstractElement webElement){
		Locator locator=webElement.getLocator();
		int size=0;
		size=findListElementSizeById(finder,locator.getLocatorById());
		if(ConstValue.ERROR==size)
			size=findListElementSizeByName(finder,locator.getLocatorByName());
		if(ConstValue.ERROR==size)
			size=findListElementSizeByClass(finder, locator.getLocatorByClass());
		if(ConstValue.ERROR==size)
			size=findListElementSizeByXpath(finder,locator.getLocatorByXpath());
		if(ConstValue.ERROR==size)
			size=findListElementSizeByCss(finder,locator.getLocatorByCSS());
		if(ConstValue.ERROR==size)
			size=findListElementSizeByLinkText(finder,locator.getLocatorByLinkText());
		if(ConstValue.ERROR==size)
			size=findListElementSizeByPartLinkText(finder,locator.getLocatorByPartialLinkText());
		if(ConstValue.ERROR==size)
			size=findListElementSizeByTagName(finder,locator.getLocatorByTagName());
		return size;
	}
	
	
	private int findListElementSizeByTagName(LocatorFinder finder,
			String tagName) {
		if(null==tagName||"".equalsIgnoreCase(tagName))
			return ConstValue.ERROR;
		try{
			return finder.findElements(By.tagName(tagName)).size();
		}catch (Exception e) {
			return ConstValue.ERROR;
		}
	}

	private int findListElementSizeByPartLinkText(LocatorFinder finder,
			String partialLinkText) {
		if(null==partialLinkText||"".equalsIgnoreCase(partialLinkText))
			return ConstValue.ERROR;
		try{
			return finder.findElements(By.partialLinkText(partialLinkText)).size();
		}catch (Exception e) {
			return ConstValue.ERROR;
		}
	}

	private int findListElementSizeByLinkText(LocatorFinder finder,
			String linkText) {
		if(null==linkText||"".equalsIgnoreCase(linkText))
			return ConstValue.ERROR;
		try{
			return finder.findElements(By.linkText(linkText)).size();
		}catch (Exception e) {
			return ConstValue.ERROR;
		}
	}

	private int findListElementSizeByCss(LocatorFinder finder,
			String CSS) {
		if(null==CSS||"".equalsIgnoreCase(CSS))
			return ConstValue.ERROR;
		try{
			return finder.findElements(By.cssSelector(CSS)).size();
		}catch (Exception e) {
			return ConstValue.ERROR;
		}
	}

	private int findListElementSizeByXpath(LocatorFinder finder,
			String xpath) {
		if(null==xpath||"".equalsIgnoreCase(xpath))
			return ConstValue.ERROR;
		try{
			return finder.findElements(By.xpath(xpath)).size();
		}catch (Exception e) {
			return ConstValue.ERROR;
		}
	}

	private int findListElementSizeByClass(LocatorFinder finder,
			String className) {
		if(null==className||"".equalsIgnoreCase(className))
			return ConstValue.ERROR;
		try{
			return finder.findElements(By.className(className)).size();
		}catch (Exception e) {
			return ConstValue.ERROR;
		}
	}

	private int findListElementSizeByName(LocatorFinder finder,
			String name) {
		if(null==name||"".equalsIgnoreCase(name))
			return ConstValue.ERROR;
		try{
			return finder.findElements(By.name(name)).size();
		}catch (Exception e) {
			return ConstValue.ERROR;
		}
	}

	private int findListElementSizeById(LocatorFinder finder, String id) {
		if(null==id||"".equalsIgnoreCase(id))
			return ConstValue.ERROR;
		try{
			return finder.findElements(By.id(id)).size();
		}catch (Exception e) {
			return ConstValue.ERROR;
		}
	}

	/**
	 * 定位普通Element类型
	 * @param finder 定位引擎
	 * @param webElement 待定位的元素
	 * @return 定位到普通Element元素，并返回此元素
	 * */
	private WebElement findElementOnly(LocatorFinder finder,AbstractElement webElement){
		Locator locator=webElement.getLocator();
		WebElement element=findElementById(finder,locator.getLocatorById());
		if(null==element)
			element=findElementByName(finder,locator.getLocatorByName());
		if(null==element)
			element=findElementByClass(finder, locator.getLocatorByClass());
		if(null==element)
			element=findElementByXpath(finder,locator.getLocatorByXpath());
		if(null==element)
			element=findElementByCss(finder,locator.getLocatorByCSS());
		if(null==element)
			element=findElementByLinkText(finder,locator.getLocatorByLinkText());
		if(null==element)
			element=findElementByPartLinkText(finder,locator.getLocatorByPartialLinkText());
		if(null==element)
			element=findElementByTagName(finder,locator.getLocatorByTagName());
		if(null==element){
			element=findElementByXpath(finder, locator.getXpathFromAttribute());
		}
		return element;
	}
	/**
	 * 定位Collection类型的元素
	 * @param finder 定位引擎
	 * @param webElement 待定位的元素
	 * @param index Collection类型元素的索引，定位到这个索引上
	 * @return 定位到的指定索引指代的Collection类型元素，并返回此元素
	 * */
	private WebElement findCollection(LocatorFinder finder,AbstractElement webElement,int index){
		Locator locator=webElement.getLocator();
		WebElement element=findCollectionById(finder,locator.getLocatorById(),index);
		if(null==element)
			element=findCollectionByName(finder,locator.getLocatorByName(),index);
		if(null==element)
			element=findCollectionByClass(finder, locator.getLocatorByClass(), index);
		if(null==element)
			element=findCollectionByXpath(finder,locator.getLocatorByXpath(),index);
		if(null==element)
			element=findCollectionByCss(finder,locator.getLocatorByCSS(),index);
		if(null==element)
			element=findCollectionByLinkText(finder,locator.getLocatorByLinkText(),index);
		if(null==element)
			element=findCollectionByPartLinkText(finder,locator.getLocatorByPartialLinkText(),index);
		if(null==element)
			element=findCollectionByTagName(finder,locator.getLocatorByTagName(),index);
		if(null==element)
			element=findCollectionByXpath(finder, locator.getXpathFromAttribute(), index);
		return element;
	}
	private WebElement findElementByTagName(LocatorFinder finder,
			String TagName) {
		if(TagName==null||"".equalsIgnoreCase(TagName))
			return null;
		try{
			return finder.findElement(By.tagName(TagName));
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findElementByPartLinkText(LocatorFinder finder,
			String PartialLinkText) {
		if(PartialLinkText==null||"".equalsIgnoreCase(PartialLinkText))
			return null;
		try{
			return finder.findElement(By.partialLinkText(PartialLinkText));
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findElementByLinkText(LocatorFinder finder,
			String LinkText) {
		if(LinkText==null||"".equalsIgnoreCase(LinkText))
			return null;
		try{
			return finder.findElement(By.linkText(LinkText));
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findElementByCss(LocatorFinder finder,
			String CSS) {
		if(CSS==null||"".equalsIgnoreCase(CSS))
			return null;
		try{
			return finder.findElement(By.cssSelector(CSS));
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findElementByXpath(LocatorFinder finder,
			String Xpath) {
		if(Xpath==null||"".equalsIgnoreCase(Xpath))
			return null;
		try{
			return finder.findElement(By.xpath(Xpath));
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findElementByName(LocatorFinder finder,
			String Name) {
		if(Name==null||"".equalsIgnoreCase(Name))
			return null;
		try{
			return finder.findElement(By.name(Name));
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findElementById(LocatorFinder finder, String Id) {
		if(Id==null||"".equalsIgnoreCase(Id))
			return null;
		try{
			return finder.findElement(By.id(Id));
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findElementByClass(LocatorFinder finder, String className) {
		if(className==null||"".equalsIgnoreCase(className))
			return null;
		try{
			return finder.findElement(By.className(className));
		}catch (Exception e) {
			return null;
		}
	}
	
	
	private WebElement findCollectionByTagName(LocatorFinder finder,
			String TagName,int index) {
		if(TagName==null||"".equalsIgnoreCase(TagName))
			return null;
		try{
			return finder.findElements(By.tagName(TagName)).get(index);
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findCollectionByPartLinkText(LocatorFinder finder,
			String PartialLinkText,int index) {
		if(PartialLinkText==null||"".equalsIgnoreCase(PartialLinkText))
			return null;
		try{
			return finder.findElements(By.partialLinkText(PartialLinkText)).get(index);
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findCollectionByLinkText(LocatorFinder finder,
			String LinkText,int index) {
		if(LinkText==null||"".equalsIgnoreCase(LinkText))
			return null;
		try{
			return finder.findElements(By.linkText(LinkText)).get(index);
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findCollectionByCss(LocatorFinder finder,
			String CSS,int index) {
		if(CSS==null||"".equalsIgnoreCase(CSS))
			return null;
		try{
			return finder.findElements(By.cssSelector(CSS)).get(index);
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findCollectionByXpath(LocatorFinder finder,
			String Xpath,int index) {
		if(Xpath==null||"".equalsIgnoreCase(Xpath))
			return null;
		try{
			return finder.findElements(By.xpath(Xpath)).get(index-1);
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findCollectionByName(LocatorFinder finder,
			String Name,int index) {
		if(Name==null||"".equalsIgnoreCase(Name))
			return null;
		try{
			return finder.findElements(By.name(Name)).get(index);
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findCollectionById(LocatorFinder finder, String Id,int index) {
		if(Id==null||"".equalsIgnoreCase(Id))
			return null;
		try{
			return finder.findElements(By.id(Id)).get(index);
		}catch (Exception e) {
			return null;
		}
	}
	private WebElement findCollectionByClass(LocatorFinder finder, String className,int index){
		if(className==null||"".equalsIgnoreCase(className))
			return null;
		try{
			return finder.findElements(By.className(className)).get(index);
		}catch (Exception e) {
			return null;
		}
	}
}
