package com.holmos.webtest.junitextentions.parameters.getter;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

import org.junit.Assert;

import com.holmos.webtest.exceptions.HolmosFailedError;
import com.holmos.webtest.junitextentions.annotations.ID;
import com.holmos.webtest.utils.HolmosAnnotationUtils;
/**
 * 对象类型参数获取器
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public final class ObjectDataGetter extends AbstractDataGetter{
	/**完整类名，带包名*/
	private String fullClassName;
	private Object obj;
	/**记录带有id注解的方法与方法的对应信息*/
	private HashMap<Integer, ArrayList<Method>>idForMethods=new HashMap<Integer, ArrayList<Method>>();
	public ObjectDataGetter(String fullClassName){
		this.fullClassName=fullClassName;
		this.obj=getObject();
		this.initIDForMethodInfo();
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public ArrayList<Object[]> getParameterByID(int id) {
		ArrayList<Method>methodsWithID=idForMethods.get(id);
		if(null==methodsWithID)return null;
		ArrayList<Object[]>result = new ArrayList<Object[]>();
		ArrayList<Object[]>reference=null;
		for(Method method:methodsWithID){
			try {
				ArrayList<Object[]> returnResult= (ArrayList<Object[]>) method.invoke(obj);
				reference=reference==null?returnResult:reference;
				volidateReturnValueTypes(reference,returnResult);
				result.addAll(returnResult);
			} catch (Exception e){}
		}
		return result;
	}

	private void volidateReturnValueTypes(ArrayList<Object[]> reference,
			ArrayList<Object[]> returnResult) {
		if(null==reference||null==returnResult)
			return;
		volidateOneMethodReturnValueTypes(reference);
		volidateOneMethodReturnValueTypes(returnResult);
		for(int i=0;i<reference.get(0).length;i++){
			if(!reference.get(0)[i].getClass().getName().equalsIgnoreCase(
					returnResult.get(0)[i].getClass().getName())){
				throw new HolmosFailedError("对象参数准备的方法:"+fullClassName+" 同一id参数类型不一致!");
			}
		}
	}

	private void volidateOneMethodReturnValueTypes(ArrayList<Object[]> reference) {
		if(reference.size()<=1)
			return;
		for(int i=1;i<reference.size();i++){
			if(reference.get(i).length!=reference.get(0).length)
				throw new HolmosFailedError("对象参数准备的方法:"+fullClassName+" 同一id参数列表长度不一样!");
			for(int j=0;j<reference.get(0).length;j++){
				if(!reference.get(i)[j].getClass().getName().equalsIgnoreCase(
						reference.get(0)[j].getClass().getName())){
					throw new HolmosFailedError("对象参数准备的方法:"+fullClassName+" 同一id参数类型不一致!");
				}
			}
		}
	}

	@Override
	public ArrayList<Object[]> getParameterByIDs(int[] ids) {
		ArrayList<Object[]>result=new ArrayList<Object[]>();
		for(int id:ids){
			result.addAll(getParameterByID(id));
		}
		return result;
	}

	@Override
	public ArrayList<Object[]> getAllParameter() {
		ArrayList<Object[]>result=new ArrayList<Object[]>();
		Set<Integer>ids=idForMethods.keySet();
		for(int id:ids){
			result.addAll(getParameterByID(id));
		}
		return result;
	}
	/**
	 * 初始化方法与id的对应信息，对方法有以下几个要求
	 * <li>方法返回信息必须是ArrayList<Object> 类型的信息</li>
	 * <li>如果两个方法具有相同的id信息，那么这两个方法返回值类型必须完全一致</li>
	 * <li></li>
	 * <li></li>
	 * */
	private void initIDForMethodInfo(){
		Set<Method>allMethods=HolmosAnnotationUtils.getMethodsWithAnnotationBase(this.obj.getClass(), ID.class, true);
		for(Method method:allMethods){
			if(volidateIDAnnotation(method)){
				int id=method.getAnnotation(ID.class).id();
				if(idForMethods.get(id)==null)
					idForMethods.put(id, new ArrayList<Method>());
				idForMethods.get(id).add(method);
			}
		}
	}
	private boolean volidateIDAnnotation(Method method){
		if(!method.getReturnType().isAssignableFrom(ArrayList.class))
			throw new HolmosFailedError("对象参数准备的方法:"+fullClassName+"."+method.getName()+"必须是ArrayList<Object[]>类型!");
		ID id=method.getAnnotation(ID.class);
		if(id.id()<=0)
			throw new HolmosFailedError("对象参数准备的方法:"+fullClassName+"."+method.getName());
		return true;
	}
	
	private Object getObject(){
		try {
			return getOnlyConstructor().newInstance();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}return null;
	}
	public Constructor<?> getOnlyConstructor() {
		Constructor<?>[] constructors;
		try {
			constructors = Class.forName(fullClassName).getConstructors();
			Assert.assertEquals(1, constructors.length);
			return constructors[0];
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}return null;
	}
}
